//-----------------------------------------------------------------------------
#ifndef PropDataH
#define PropDataH PropDataH
//-----------------------------------------------------------------------------
#include <mvIMPACT_CPP/mvIMPACT_acquire.h>
#include <wxPropGrid/Include/propgrid.h>
#ifndef SWIG
#	include <string>
#endif

#ifndef wxPG_EXTRAS_DECL
#	ifdef __WXPYTHON__
		// The Python wrapper is a dll
#		define wxPG_EXTRAS_DECL WXDLLIMPEXP_PG
#	else
		// For a normal application
#		define wxPG_EXTRAS_DECL
#	endif
#endif

#ifndef SWIG
#	include <apps/Common/wxAbstraction.h>
#endif

class PropTree;

//------------------------------------------------------------------------------
class PropData
//------------------------------------------------------------------------------
{
public:
	WXDLLIMPEXP_PG enum ECtrl
	{
		_ctrlStatic,
		_ctrlSpinner,
		_ctrlEdit,
		_ctrlCombo,
		_ctrlMultiChoiceSelector,
		_ctrlFileSelector,
		_ctrlDirSelector
	};
	WXDLLIMPEXP_PG explicit							PropData( mvIMPACT::acquire::HOBJ hObj );
	WXDLLIMPEXP_PG virtual						   ~PropData( void ) {}
	WXDLLIMPEXP_PG virtual void						OnExpand( void ) {}
	WXDLLIMPEXP_PG virtual void						UpdatePropData( void ) {}
	WXDLLIMPEXP_PG virtual void						EnsureValidGridItem( const PropTree *pPropTree, wxPGId parentGridComponent, bool* boModified = 0  ) = 0;
	WXDLLIMPEXP_PG void								UpdateGridItem( const PropTree *pPropTree, bool boDisplayDebugInfo, bool boDisplayInvisibleComponents, bool *boModified );
	WXDLLIMPEXP_PG virtual void						Update( const PropTree *pPropTree, bool boDisplayDebugInfo, unsigned int actChangedCount, unsigned int actAttrChangedCount ) const = 0;
	WXDLLIMPEXP_PG void								InvalidatePropGridItem( void ) { m_GridItemId = wxNullProperty; }
	WXDLLIMPEXP_PG mvIMPACT::acquire::Component		Component( void ) const { return m_Component; }
	WXDLLIMPEXP_PG bool								HasChanged( void ) const { return m_Component.changedCounter() != m_lastChangedCounter; }
	WXDLLIMPEXP_PG ECtrl							Type( void ) const { return m_Type; }
	WXDLLIMPEXP_PG wxPGId							GetGridItem( void ) const { return m_GridItemId; }

protected:
	void							AppendComponentInfo( wxString& info, unsigned int actChangedCount, unsigned int actAttrChangedCount ) const;
	static const wxColour			m_IS_DEFAULT_VALUE_COLOUR;
	const mvIMPACT::acquire::Component	m_Component;
	unsigned int					m_lastChangedCounter;
	unsigned int					m_lastChangedCounterAttr;
	ECtrl							m_Type;
	wxPGId							m_GridItemId;
	wxPropertyGrid*					m_pParentGrid;
};

//------------------------------------------------------------------------------
class MethodObject : public PropData
//------------------------------------------------------------------------------
{
public:
	WXDLLIMPEXP_PG explicit				MethodObject( mvIMPACT::acquire::HOBJ hObj );
	WXDLLIMPEXP_PG wxString				Call( void ) const;
	WXDLLIMPEXP_PG const wxString&		Params( void ) const { return m_Params; }
	WXDLLIMPEXP_PG void					UpdatePropData( void );
	WXDLLIMPEXP_PG void					Update( const PropTree *pPropTree, bool boDisplayDebugInfo, unsigned int actChangedCount, unsigned int actAttrChangedCount ) const;
	WXDLLIMPEXP_PG void					EnsureValidGridItem( const PropTree *pPropTree, wxPGId parentItem, bool* boModified = 0  );
private:
	wxString							m_Params;
	wxString							m_FriendlyName;
};

//------------------------------------------------------------------------------
class ListObject : public PropData
//------------------------------------------------------------------------------
{
public:
	WXDLLIMPEXP_PG explicit					ListObject( mvIMPACT::acquire::HOBJ hObj, const char *title = 0 );
	WXDLLIMPEXP_PG bool						IsExpanded( void ) const { return m_boExpanded; }
	WXDLLIMPEXP_PG void						OnExpand( void );
	WXDLLIMPEXP_PG void						EnsureValidGridItem( const PropTree *pPropTree, wxPGId parentItem, bool* boModified = 0  );
	WXDLLIMPEXP_PG void						Update( const PropTree *pPropTree, bool boDisplayDebugInfo, unsigned int actChangedCount, unsigned int actAttrChangedCount ) const;
	WXDLLIMPEXP_PG static const wxColour	BACKGROUND_COLOR;
private:
	//bool							GetColour( std::string& text, wxColour& colour );
	bool							m_boExpanded;
	const wxString					m_Title;
	//static std::string			m_colourKey;
};

//------------------------------------------------------------------------------
class PropertyObject : public PropData
//------------------------------------------------------------------------------
{
public:
	WXDLLIMPEXP_PG explicit			PropertyObject( mvIMPACT::acquire::HOBJ hObj, int index = 0 );
	WXDLLIMPEXP_PG void				UpdatePropData( void );
	WXDLLIMPEXP_PG void				EnsureValidGridItem( const PropTree *pPropTree, wxPGId parentItem, bool* boModified = 0  );
	WXDLLIMPEXP_PG int				GetIndex( void ) const { return m_Index; }
	WXDLLIMPEXP_PG void				Update( const PropTree *pPropTree, bool boDisplayDebugInfo, unsigned int actChangedCount, unsigned int actAttrChangedCount ) const;
private:
	wxString						GetCurrentValueAsString( void ) const;
	void							GetTransformedDict( wxPGChoices& soc ) const;
	static bool						ShouldBeSpinControl( const Property& prop );
	void							UpdateLabel( bool boDisplayDebugInfo, unsigned int actChangedCount, unsigned int actAttrChangedCount ) const;
	void							WritePropVal( const std::string& value ) const;
	const int						m_Index;
	const bool						m_boVectorAsList;
};

//------------------------------------------------------------------------------
class VectorPropertyObject : public PropData
//------------------------------------------------------------------------------
{
public:
	WXDLLIMPEXP_PG explicit			VectorPropertyObject( mvIMPACT::acquire::HOBJ hObj );
	WXDLLIMPEXP_PG				   ~VectorPropertyObject();
	WXDLLIMPEXP_PG bool				IsExpanded( void ) const { return m_boExpanded; }
	WXDLLIMPEXP_PG void				OnExpand( void );
	WXDLLIMPEXP_PG PropertyObject*	GetVectorItem( int index );
	WXDLLIMPEXP_PG void				EnsureValidGridItem( const PropTree *pPropTree, wxPGId parentItem, bool* boModified = 0 );
	WXDLLIMPEXP_PG void				RemoveValue( unsigned int index );
	WXDLLIMPEXP_PG void				Resize( void );
	WXDLLIMPEXP_PG void				Update( const PropTree *pPropTree, bool boDisplayDebugInfo, unsigned int actChangedCount, unsigned int actAttrChangedCount ) const;
private:
	bool							m_boExpanded;
	std::vector<PropertyObject*>	m_VectorItems;
};

#endif // PropDataH
